import java.io.*;

public class Warenkorb {

  private Artikel anchor; //Listenanker als Attribut

  public Warenkorb() {  //Konstruktor
    anchor = null;      //Leerverweis setzen, da Warenkorb initial leer
  }


  /* ----- neuen Datensatz einfuegen am Listenanfang */

  public void fuegeArtikelHinzu(String name, long code, float stueckpreis) {
    if ((name.length() < 1) || (code < 1) || (stueckpreis < 0)) {
      return; //Plausibilitaetspruefung. Datensatz nur bei gueltigen Daten anlegen
    }
    if (anchor == null) {
      anchor = new Artikel(name, code, stueckpreis, null); //erster Eintrag
    } else {
      anchor = new Artikel(name, code, stueckpreis, anchor); //Folgeeintrag
    }
  }


  /* ----- Anzahl Datensaetze in Liste */

  public int anzahlArtikel() {
    Artikel h = anchor; //sog. Iterator, der von Datensatz zu Datensatz hangelt
    int i = 0;

    while (h != null) {
      i++;
      h = h.next; //Springe im Speicher zum naechsten Datensatz 
    }
    return i;
  }


  /* ----- Vom Datensatz mit gegebenem Index den Produktnamen zurueckgeben */

  public String gibProduktname(int index) {
    Artikel h = anchor;
    int i = 0; 

    while (h != null) {
      if (i == index) {
        return h.produktname;
      }
      h = h.next;
      i++;
    }
    return "";
  }


  /* ----- Vom Datensatz mit gegebenem Index den Produktcode zurueckgeben */

  public long gibProduktcode(int index) {
    Artikel h = anchor;
    int i = 0; 

    while (h != null) {
      if (i == index) {
        return h.produktcode;
      }
      h = h.next;
      i++;
    }
    return 0;
  }


  /* ----- Vom Datensatz mit gegebenem Index den Stueckpreis zurueckgeben */

  public float gibStueckpreis(int index) {
    Artikel h = anchor;
    int i = 0; 

    while (h != null) {
      if (i == index) {
        return h.preis;
      }
      h = h.next;
      i++;
    }
    return 0.0f;
  }


  /* ----- Alle Datensaetze der Liste anzeigen */

  public void zeigeWarenkorbAn() {
    System.out.printf("\nGesamte Liste:\n");
    System.out.printf("Nr.                    Produkt    Produktcode  Stueckpreis\n");
    System.out.printf("-------------------------------------------------------------\n");
    for(int i = 0; i < anzahlArtikel(); i++) {
      System.out.printf("%3d  %25s     %10d         %.2f\n", i, gibProduktname(i), gibProduktcode(i), gibStueckpreis(i));
    }  
  }


  /* ----- Alle Datensaetze in Textdatei speichern */

  public void speichereWarenkorb(String dateiname) throws IOException {
    if (dateiname.length() < 1) {
      return;
    }
    File f = new File(dateiname);
    BufferedWriter bw = new BufferedWriter(new FileWriter(f));
    String dateiinhalt = "";

    for(int i = 0; i < anzahlArtikel(); i++) {
      String zeile = "" + gibProduktname(i) + ", ";
      zeile = zeile + String.valueOf(gibProduktcode(i)) + ", ";
      zeile = zeile + String.valueOf(gibStueckpreis(i)) + "\n";
      dateiinhalt = dateiinhalt + zeile;
      zeile = "";
    }
    bw.write(dateiinhalt);
    bw.close();
  }


  /* ----- neuen Datensatz einfuegen an Indexposition i mit 0 <= i <= anzahlArtikel() */

  public void fuegeArtikelHinzuAnPosition(int index, String name, long code, float stueckpreis) {
    Artikel a = anchor;

    int i = 0;
    
    if ((index < 0) || (index > anzahlArtikel())) {
      return;
    } /* ungueltiger Index */
    if (index == 0) {
      fuegeArtikelHinzu(name, code, stueckpreis); /* Einfuegen am Anfang */
      return;
    }
    while (i < index - 1)      /* Bis zum Datensatz vor einzufuegendem laufen */
    {
      i++;
      a = a.next;
    }
    Artikel h = new Artikel(name, code, stueckpreis, a.next); /* Neuen Artikel als Objekt anlegen */
    a.next = h;               /* Verweis Vorgaenger auf neuen Datensatz setzen */
  }


  /* ----- Datensatz mit gegebenem Index loeschen */

  public void loescheArtikel(int index) {
    Artikel h = anchor;
    int i = 0;

    if ((index < 0) || (index >= anzahlArtikel())) {
      return; /* ungueltiger Index */
    }
    if (index == 0) { /* Listenelement am Listenanfang loeschen */
      anchor = anchor.next;
      return;
    }
    while (i < index - 1) {
      i++;
      h = h.next;
    }
    h.next = h.next.next;
  }


  /* ----- Liste aufsteigend nach Stueckpreis sortieren mit Insertionsort */

  public void insertsort() {
    String name;
    long code;
    float stueckpreis, stueckpreis2;
    int k = 1; /* Index, bis wohin schon sortiert */
    int r;
  
    while (k < anzahlArtikel()) {
      name = gibProduktname(k);
      code = gibProduktcode(k);
      stueckpreis = gibStueckpreis(k);
      loescheArtikel(k); //Datensatz mit index k aus Liste entfernen
      r = 0;
      do
      {
        stueckpreis2 = gibStueckpreis(r);
        r++;
      } while ((r-1 < k) && (stueckpreis > stueckpreis2));
      fuegeArtikelHinzuAnPosition(r-1, name, code, stueckpreis);
      k++;
    }
  }
} //class

